﻿#include "pch.h"
#include "TestTextRendererRenderer.h"
#include "Common/DirectXHelper.h"

using namespace TestTextRenderer;

using namespace D2D1;
using namespace Microsoft::WRL;

TestTextRendererRenderer::TestTextRendererRenderer(const std::shared_ptr<DX::DeviceResources>& deviceResources) : 
    m_deviceResources(deviceResources)
{
    // Create device independent resources
    DX::ThrowIfFailed(
        m_deviceResources->GetD2DFactory()->CreateDrawingStateBlock(&m_stateBlock)
        );

    ComPtr<IDWriteTextFormat> textFormat;

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateTextFormat(
            L"Times New Roman",
            nullptr,
            DWRITE_FONT_WEIGHT_NORMAL,
            DWRITE_FONT_STYLE_NORMAL,
            DWRITE_FONT_STRETCH_NORMAL,
            3 * 96.0f,
            L"en-US",
            &textFormat)
        );

    DX::ThrowIfFailed(
        m_deviceResources->GetDWriteFactory()->CreateTextLayout(
            L"DX",
            2,
            textFormat.Get(),
            std::numeric_limits<float>::infinity(),
            std::numeric_limits<float>::infinity(),
            &m_textLayout)
        );

    DWRITE_TEXT_RANGE textRange;
    textRange.startPosition = 0;
    textRange.length = 2;

    DX::ThrowIfFailed(
        m_textLayout->SetUnderline(true, textRange)
        );

    DX::ThrowIfFailed(
        m_textLayout->SetStrikethrough(true, textRange)
        );

    DX::ThrowIfFailed(
        m_textLayout->GetMetrics(&m_textMetrics)
        );

    // Instantiate BasicTextRenderer
    m_basicTextRenderer = new BasicTextRenderer();
    CreateDeviceDependentResources();
}

void TestTextRendererRenderer::CreateDeviceDependentResources()
{
    ID2D1DeviceContext1 * context = m_deviceResources->GetD2DDeviceContext();

    DX::ThrowIfFailed(
        context->CreateSolidColorBrush(ColorF(ColorF::Black), &m_blackBrush)
        );

    DX::ThrowIfFailed(
        context->CreateSolidColorBrush(ColorF(1.0f, 0, 0, 0.5f), &m_overlayBrush)
        );
}

void TestTextRendererRenderer::ReleaseDeviceDependentResources()
{
    m_blackBrush.Reset();
}

// Updates the text to be displayed.
void TestTextRendererRenderer::Update(DX::StepTimer const& timer)
{
}

// Renders a frame to the screen.
void TestTextRendererRenderer::Render()
{
    ID2D1DeviceContext* context = m_deviceResources->GetD2DDeviceContext();
    Windows::Foundation::Size logicalSize = m_deviceResources->GetLogicalSize();

    context->SaveDrawingState(m_stateBlock.Get());
    context->BeginDraw();
    context->Clear(ColorF(ColorF::AliceBlue));

    // Position in the center 
    Matrix3x2F screenTranslation = Matrix3x2F::Translation(
        (logicalSize.Width - m_textMetrics.width) / 2,
        (logicalSize.Height - m_textMetrics.height) / 2);

    context->SetTransform(screenTranslation * m_deviceResources->GetOrientationTransform2D());

    D2D1_POINT_2F origin = D2D1::Point2F();

    // Display text normally
    context->DrawTextLayout(origin,
                            m_textLayout.Get(),
                            m_blackBrush.Get());
    
    // Display text using custom renderer
    DrawingContext drawingContext(context, 
                                  m_overlayBrush.Get());

    DX::ThrowIfFailed(
        m_textLayout->Draw(&drawingContext, 
                           m_basicTextRenderer.Get(), 
                           origin.x, origin.y)
        );
       
    // Ignore D2DERR_RECREATE_TARGET here. This error indicates that the device
    // is lost. It will be handled during the next call to Present.
    HRESULT hr = context->EndDraw();
    if (hr != D2DERR_RECREATE_TARGET)
    {
        DX::ThrowIfFailed(hr);
    }

    context->RestoreDrawingState(m_stateBlock.Get());
}

